@@ -23,7 +23,7 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include "system.h"
 #include "cpplib.h"
 #include "cpphash.h"
-#include "prefix.h"
+// #include "prefix.h"
 #include "intl.h"
 #include "mkdeps.h"
 #include "cppdefault.h"
